# frozen_string_literal: true

require 'thor'
require 'date'
require_relative 'commands/layout'
require_relative 'commands/field'
require_relative 'i18n'
require_relative '../schema'

module Sutty
  module Cli
    # Handle the application command line parsing
    # and the dispatch to various command objects
    #
    # @api public
    class CLI < Thor
      # Error raised by this runner
      Error = Class.new(StandardError)

      desc I18n.t('version.desc.example'), I18n.t('version.desc.text')
      def version
        require_relative 'version'
        puts "v#{Sutty::Cli::VERSION}"
      end
      map %w(--version -v) => :version

      desc I18n.t('post.desc.example'), I18n.t('post.desc.text')
      method_option :help, aliases: '-h', type: :boolean, desc: I18n.t('help')

      method_option :layout,
                    aliases: '-l',
                    type: :string,
                    desc: I18n.t('post.flags.layout'),
                    required: true,
                    enum: Sutty::Cli::Commands::Layout.layouts

      method_option :title,
                    aliases: '-t',
                    type: :string,
                    desc: I18n.t('post.flags.title')

      method_option :date,
                    aliases: '-d',
                    type: :string,
                    desc: I18n.t('post.flags.date'),
                    default: Date.today.to_s

      # TODO: Bring locales from Jekyll configuration
      method_option :locale,
                    aliases: '-L',
                    type: :string,
                    desc: I18n.t('post.flags.locale'),
                    default: 'posts'

      method_option :content,
                    aliases: '-c',
                    type: :string,
                    desc: I18n.t('post.flags.content'),
                    default: 'short',
                    enum: %w[short long]

      def post
        if options[:help]
          invoke :help, ['post']
        else
          require_relative 'commands/post'
          Sutty::Cli::Commands::Post.new(options).execute
        end
      end

      desc I18n.t('container.desc.example'), I18n.t('container.desc.text')
      method_option :help, aliases: '-h', type: :boolean, desc: I18n.t('help')
      def container(name)
        if options[:help]
          invoke :help, ['container']
        else
          require_relative 'commands/container'
          Sutty::Cli::Commands::Container.new(name, options).execute
        end
      end

      desc I18n.t('field.desc.example'), I18n.t('field.desc.text')
      long_desc I18n.t('field.desc.long')

      method_option :help,
                    aliases: '-h',
                    type: :boolean,
                    desc: I18n.t('help')

      method_option :before,
                    type: :string,
                    desc: I18n.t('field.flags.before'),
                    default: 'order'

      method_option :force,
                    type: :boolean,
                    desc: I18n.t('field.flags.force'),
                    default: false

      method_option :layout,
                    aliases: '-l',
                    type: :string,
                    desc: I18n.t('field.flags.layout'),
                    required: true,
                    enum: Sutty::Cli::Commands::Layout.layouts

      method_option :type,
                    aliases: '-t',
                    type: :string,
                    desc: I18n.t('field.flags.type'),
                    required: true,
                    enum: Sutty::Schema.field_types.map(&:to_s)

      method_option :required,
                    aliases: '-r',
                    type: :boolean,
                    desc: I18n.t('field.flags.required')

      method_option :private,
                    aliases: '-p',
                    type: :boolean,
                    desc: I18n.t('field.flags.private')

      method_option :inverse,
                    aliases: '-i',
                    type: :string,
                    desc: I18n.t('field.flags.inverse'),
                    required: Sutty::Cli::Commands::Field.inverse_required?

      method_option :filter,
                    aliases: '-F',
                    type: :string,
                    desc: I18n.t('field.flags.filter')

      method_option :value,
                    aliases: '-v',
                    type: :string,
                    desc: I18n.t('field.flags.value'),
                    required: Sutty::Cli::Commands::Field.value_required?

      def field(name)
        if options[:help]
          invoke :help, ['field']
        else
          Sutty::Cli::Commands::Field.new(name, options).execute
        end
      end

      desc I18n.t('layout.desc.example'), I18n.t('layout.desc.text')
      method_option :help, aliases: '-h', type: :boolean, desc: I18n.t('help')
      method_option :schema_only,
                    aliases: '-s',
                    type: :boolean,
                    default: false,
                    desc: I18n.t('layout.flags.schema_only')
      def layout(name)
        if options[:help]
          invoke :help, ['layout']
        else
          Sutty::Cli::Commands::Layout.new(name, options).execute
        end
      end

      desc I18n.t('theme.desc.example'), I18n.t('theme.desc.text')
      method_option :help, aliases: '-h', type: :boolean, desc: I18n.t('help')
      def theme(name)
        if options[:help]
          invoke :help, ['theme']
        else
          require_relative 'commands/theme'
          Sutty::Cli::Commands::Theme.new(name, options).execute
        end
      end
    end
  end
end
