# frozen_string_literal: true

require_relative '../command'
require_relative '../../schema'
require 'tty-file'
require 'tty-logger'
require 'active_support/core_ext/hash/keys'
require 'active_support/core_ext/object/blank'

class Hash
  # Agrega una llave antes de otra
  #
  # @param [Any] La llave a buscar
  # @param [Array] Array de dos elementos, el primero es la llave, el segundo el valor.
  # @return [Hash]
  # @see {https://stackoverflow.com/a/17251424}
  def insert_before(key, kvpair)
    arr = to_a
    pos = arr.index(arr.assoc(key))

    if pos
      arr.insert(pos, kvpair)
    else
      arr << kvpair
    end

    replace Hash[arr]
  end
end

module Sutty
  module Cli
    module Commands
      class Field < Sutty::Cli::Command
        attr_reader :name, :options

        INVERSE_OF = {
          'has_many' => 'belongs_to',
          'belongs_to' => 'has_many',
          'has_and_belongs_to_many' => 'has_and_belongs_to_many'
        }.freeze

        # Campos que requieren especificar la relación inversa
        INVERSE_REQUIRED = %w[has_many belongs_to has_and_belongs_to_many].freeze
        # Opciones que necesitan un valor
        VALUE_REQUIRED = %w[-F --filter].freeze
        # Nombres de campos que no se pueden usar
        RESERVED_FIELDS = %w[url date path slug ext].freeze

        def initialize(name, options)
          @name = name
          @options = options.to_h.transform_keys(&:to_sym)
          @options[:before] ||= 'order'

          unless @options[:layout]
            binding.pry
            raise ArgumentError, 'layout option is missing'
          end
        end

        # TODO: Generar la relación inversa en el otro layout, a partir
        # del filtro.
        #
        # @return [Boolean]
        def execute(input: $stdin, output: $stdout)
          @output = output

          # Crea el layout si no existe
          unless File.exist? layout_file
            Sutty::Cli::Commands::Layout.new(options[:layout], {}).execute
          end

          if RESERVED_FIELDS.include? name
            logger.info I18n.t('field.errors.reserved', name: name)
            return false
          end

          if layout_contents.key?(name) && !options[:force]
            logger.info I18n.t('field.errors.already_exists', name: name)
            return false
          end

          if INVERSE_REQUIRED.include?(options[:type]) && options[:inverse].blank? && options[:filter].blank?
            logger.info I18n.t('field.errors.inverse_required', name: name)
            return false
          end

          layout_contents.insert_before options[:before],
            Sutty::Schema.create(name: name, **options).to_a.flatten

          # TODO: Mostrar el mismo mensaje de append_to_file
          TTY::File.remove_file layout_file
          TTY::File.create_file layout_file, YAML.dump(layout_contents.deep_stringify_keys)

          # Agrega la relación inversa, creando el layout también.
          if INVERSE_REQUIRED.include?(options[:type])
            # XXX: No usar force aquí
            Sutty::Cli::Commands::Field.new(options[:inverse],
                                            layout: options[:value],
                                            type: inverse_of(options[:type]),
                                            filter: :layout,
                                            value: options[:layout],
                                            inverse: name).execute(output: output)
          end

          layout_contents.key? name
        end

        # Determina si la relación inversa es necesaria
        def self.inverse_required?
          ARGV.any? { |f| INVERSE_REQUIRED.include? f }
        end

        # Determina si el valor es necesario
        def self.value_required?
          ARGV.any? { |f| VALUE_REQUIRED.include? f }
        end

        private

        def layout_file
          @data_file ||= File.join('_data', 'layouts', options[:layout] + '.yml')
        end

        def layout_contents
          @layout_contents ||= YAML.load(File.read(layout_file))
        end

        def logger
          @logger ||= TTY::Logger.new(output: @output)
        end

        # Devuelve la relación inversa
        #
        # @return [String]
        def inverse_of(type)
          INVERSE_OF[type]
        end
      end
    end
  end
end
