# frozen_string_literal: true

require_relative '../command'
require_relative '../../schema'
require 'tty-file'
require 'yaml'
require 'active_support/core_ext/hash/keys'

module Sutty
  module Cli
    module Commands
      # Crea un layout de Jekyll y su layout correspondiente en Sutty.
      #
      # _data/layouts/NAME.yml: Definición del layout para Sutty
      # _layouts/NAME.yml: La plantilla Liquid para Jekyll (opcional)
      class Layout < Sutty::Cli::Command
        attr_reader :name

        # @param [String] Nombre del layout
        # @param [Hash] Opciones
        def initialize(name, options = {})
          @name = name
          @options = options
        end

        # Crea los dos archivos sin pisarlos si ya existen
        #
        # @return [Boolean] Si el archivo fue creado o no
        def execute(input: $stdin, output: $stdout)
          TTY::File.create_file data_layout, YAML.dump(default_fields.deep_stringify_keys)

          unless @options[:schema_only]
            TTY::File.create_file html_layout, "---\nlayout: default\n---\n\n"
          end

          File.exist? data_layout
        end

        # Devuelve el listado de layouts disponibles
        #
        # @return [Array]
        def self.layouts
          @layouts ||= Dir.glob("#{File.join('_data', 'layouts')}/*.yml").map do |d|
            File.basename d, '.yml'
          end.sort
        end

        private

        def default_fields
          @default_fields ||= [
            Sutty::Schema.string_field(name: :title, required: true),
            Sutty::Schema.order_field(name: :order),
            Sutty::Schema.boolean_field(name: :draft)
          ].inject(&:merge)
        end

        # @return [String]
        def data_layout
          @data_layout ||= File.join('_data', 'layouts', "#{name}.yml")
        end

        # @return [String]
        def html_layout
          @html_layout ||= File.join('_layouts', "#{name}.html")
        end
      end
    end
  end
end
