# frozen_string_literal: true

require 'jekyll/utils'

module Sutty
  # Genera definiciones de campos para Sutty
  class Schema
    class << self
      ATTRIBUTES = %i[type required private writable inverse filter]

      # Definición base.
      #
      # Se eliminan los valores que son falsos o están vacíos.
      #
      # @return [Hash]
      def base(**args)
        validate_field_type(args[:type])

        args.slice(*ATTRIBUTES).transform_keys(&:to_s).reject do |_, v|
          v == false || v.nil?
        end.merge(t(**args))
      end

      def create(type:, **args)
        validate_field_type(type)

        public_send :"#{type}_field", **args
      end

      # Aplica traducciones
      #
      # @param :type [String]
      # @param :name [String]
      # @return [Hash] traducciones
      def t(**args)
        require_relative 'drop_stub'

        I18n.available_locales.map do |locale|
          I18n.with_locale(locale) do
            %i[name type].map do |translation|
              translation = I18n.t("fields.common_by_#{translation}.#{args[translation]}")

              next if translation.is_a? String

              translation
            end.compact.first
          end
        end.compact.reduce({}) do |t, l|
          Jekyll::Utils.deep_merge_hashes(t, l)
        end
      end

      # Texto de una sola línea
      #
      # @return [Hash]
      def string_field(name:, **args)
        { name => base(type: 'string', name: name, **args) }
      end

      # Texto de varias líneas
      #
      # @return [Hash]
      def text_field(name:, **args)
        { name => base(type: 'text', name: name, **args) }
      end

      # Números
      #
      # @return [Hash]
      def number_field(name:, **args)
        { name => base(type: 'number', name: name, **args) }
      end

      # URL
      #
      # @return [Hash]
      def url_field(name:, **args)
        { name => base(type: 'url', name: name, **args) }
      end

      # E-mail
      #
      # @return [Hash]
      def email_field(name:, **args)
        { name => base(type: 'email', name: name, **args) }
      end

      # Teléfono
      #
      # @return [Hash]
      def tel_field(name:, **args)
        { name => base(type: 'tel', name: name, **args) }
      end

      # Color
      #
      # @return [Hash]
      def color_field(name:, **args)
        { name => base(type: 'color', name: name, **args) }
      end

      # SKU
      #
      # @return [Hash]
      def sku_field(name:, **args)
        { name => base(type: 'sku', name: name, **args) }
      end

      # Precio.  Los precios se escriben una sola vez.
      #
      # @return [Hash]
      def price_field(name:, **args)
        { name => base(type: 'price', name: name, writable: 'once', **args) }
      end

      # Permalinks
      #
      # @return [Hash]
      def permalink_field(name:, **args)
        { name => base(type: 'permalink', name: name, **args) }
      end

      # Orden
      #
      # @return [Hash]
      def order_field(name:, **args)
        { name => base(type: 'order', name: name, **args) }
      end

      # Verdadero/falso
      #
      # @return [Hash]
      def boolean_field(name:, **args)
        { name => base(type: 'boolean', name: name, **args) }
      end

      # Contenido
      #
      # @return [Hash]
      def content_field(name:, **args)
        { name => base(type: 'content', name: name, **args) }
      end

      # HTML
      #
      # @return [Hash]
      def html_field(name:, **args)
        { name => base(type: 'html', name: name, **args) }
      end

      # Oculto
      #
      # @return [Hash]
      def hidden_field(name:, **args)
        { name => base(type: 'hidden', name: name, **args) }
      end

      # Fecha
      #
      # @return [Hash]
      def date_field(name:, **args)
        { name => base(type: 'date', name: name, **args) }
      end

      # Coordenadas geográficas
      #
      # @return [Hash]
      def geo_field(name:, **args)
        { name => base(type: 'geo', name: name, **args) }
      end

      # Archivo
      #
      # @return [Hash]
      def file_field(name:, **args)
        { name => base(type: 'file', name: name, **args) }
      end

      # Imagen
      #
      # @return [Hash]
      def image_field(name:, **args)
        { name => base(type: 'image', name: name, **args) }
      end

      # Array
      #
      # @return [Hash]
      def array_field(name:, **args)
        { name => base(type: 'array', name: name, **args) }
      end

      # Relacionados sin relación recíproca
      #
      # @return [Hash]
      def related_posts_field(name:, **args)
        { name => base(type: 'related_posts', name: name, **args) }
      end

      # Relación de muchos a uno, la relación recíproca es has_many
      #
      # @return [Hash]
      def belongs_to_field(name:, filter:, value:, **args)
        { name => base(type: 'belongs_to', name: name, filter: { filter => value }, **args) }
      end

      # Relación de uno a muchos, la relación recíproca es belongs_to
      #
      # @return [Hash]
      def has_many_field(name:, filter:, value:, **args)
        { name => base(type: 'has_many', name: name, filter: { filter => value }, **args) }
      end

      # Relación de muchos a muchos, la relación recíproca es
      # has_and_belongs_to_many.
      #
      # @return [Hash]
      def has_and_belongs_to_many_field(name:, filter:, value:, **args)
        { name => base(type: 'has_and_belongs_to_many', name: name, filter: { filter => value }, **args) }
      end

      # Los tipos de campos válidos son los que esta clase define como
      # métodos terminados en _field
      #
      # @return [Array]
      def field_types
        @field_types ||= self.public_methods.select do |m|
          m.to_s.end_with? '_field'
        end.map do |m|
          m.to_s.sub(/_field\z/, '').to_sym
        end.sort
      end

      # Valida si el tipo de campo está definido
      #
      # @param [String,Symbol]
      # @return [Boolean]
      def field_type?(type)
        self.respond_to? :"#{type}_field"
      end

      # Devuelve un error si el tipo del campo no es válido
      #
      # @return [Nil]
      def validate_field_type(type)
        raise ArgumentError, "#{type} must be one of #{field_types.map(&:to_s).join(', ')}" unless field_type? type
      end
    end
  end
end
